# Filters added to this controller apply to all controllers in the application.
# Likewise, all the methods added will be available for all controllers.

# TODO: When accepting XML or form-encoded key-value pairs, trigger errors if there are surplus parameters
# TODO: Move http_* methods to an HttpResponse module
# TODO: Use ActionController::MethodNotAllowed exception class for 405 errors
# TODO: Send HTML docs by default instead of text
# FIXME: Sending invalid XML dies ungracefully
class ApplicationController < ActionController::Base
  helper :all # include all helpers, all the time
  
  helper_method :path_with_referer

  # See ActionController::RequestForgeryProtection for details
  # Uncomment the :secret if you're not using the cookie session store
  #
  # Disabled as un-safe actions do not rely on session-based authentication
  # protect_from_forgery # :secret => 'd9f4f9c14f51a69a14e7d3fca6c3af5e'
  
  # See ActionController::Base for details 
  # Uncomment this to filter the contents of submitted sensitive data parameters
  # from your application log (in this case, all fields with names like "password"). 
  filter_parameter_logging :password
  
  # before_filter :check_allowed_methods
  before_filter :check_for_authentication
  before_filter :init_common_vars
  
  around_filter :catch_exceptions
  
  private
  def catch_exceptions
    yield
  rescue ActionController::RoutingError, ActionController::UnknownAction
    http_not_found
  rescue ActionController::MethodNotAllowed
    http_method_not_allowed
  rescue Exception => e
    http_internal_server_error(e)
  end
  
  def init_common_vars
    @html = {}
    ActionMailer::Base.default_url_options[:host] ||= request.host
  end
  
  protected
  def http_ok(options = {})
    defaults = { :text => 'OK', :content_type => 'text/plain' }
    options = defaults.merge(options)
    render :text => options[:text], :status => 200, :content_type => options[:content_type]
  end
  
  def http_created(options = {})
     render options.merge({ :status => 201 })
  end
  
  def http_moved_permanently(location)
    render :nothing => true, :status => 301, :location => location, :content_type => 'text/plain'
  end
  
  def http_bad_request(errors = nil)
    if errors.nil?
      render :nothing => true, :status => 400, :content_type => 'text/plain'
    else
      unless errors.is_a?(ActiveRecord::Errors)
        raise ArgumentError, "Expected ActiveRecord::Errors but got #{errors.class.to_s}"
      end
      render :text => errors.to_xml, :status => 400, :content_type => 'application/xml'
    end
  end

  def http_forbidden(message = 'Forbidden')
    respond_to do |format|
      format.xml do
        render :text => message, :status => 403, :content_type => 'text/plain'
      end
      format.html do
        @message = message
        render :template => '/application/errors/403', :status => 403
      end
    end
  end

  def http_not_found
    respond_to do |format|
      format.xml do
        render :text => 'Not found', :status => 404, :content_type => 'text/plain'
      end
      format.html do
        render :template => '/application/errors/404', :status => 404
      end
    end
  end

  def http_method_not_allowed(allowed = nil)
    response.headers["Allow"] = allowed.collect { |method| method.to_s.upcase }.join(', ') unless allowed.nil?
    render :nothing => true, :status => 405, :content_type => 'text/plain'
  end

  def http_conflict(errors)
    unless errors.is_a?(ActiveRecord::Errors)
      raise ArgumentError, "Expected ActiveRecord::Errors but got #{errors.class.to_s}"
    end
    render :text => errors.to_xml, :status => 409, :content_type => 'application/xml'
  end
  
  # Sends HTTP 422 status code, a WebDAV extension also used by
  # ActiveResource to anticipate presence of <errors> XML in response
  def http_unprocessable_entity(errors)
    unless errors.is_a?(ActiveRecord::Errors)
      raise ArgumentError, "Expected ActiveRecord::Errors but got #{errors.class.to_s}"
    end
    render :text => errors.to_xml, :status => 422, :content_type => 'application/xml'
  end
  
  def http_internal_server_error(error = nil)
    logger.error("Internal server error: " + error.inspect + "\nBacktrace:\n" + error.backtrace.join("\n"))
    respond_to do |format|
      format.xml do
        if errors
          render :xml => error.to_xml, :status => 500
        else
          render :text => 'Internal server error', :status => 500, :content_type => 'text/plain'
        end
      end
      format.html do
        @error = error if self.class.consider_all_requests_local
        render :template => '/application/errors/500', :status => 500
      end
    end
  end
  
  def check_for_authentication
    if session[:authenticated_user_id]
      if user = User.find_by_id(session[:authenticated_user_id])
        @authenticated_user = user
        logger.debug("Authenticated in session as user: #{user.name}")
      else
        session.delete(:authenticated_user_id)
      end
    else
      authenticate_with_http_basic do |user_name, password|
        if user = User.authenticate(user_name, password)
          @authenticated_user = user
          logger.debug("Authenticated with HTTP Basic as user: #{user_name}")
        else
          logger.debug("Failed HTTP Basic authentication as user: #{user_name}")
          request_authentication
          return false
        end
      end
    end
  end
        
  # TODO: Move HTTP auth realm name out into configuration file (environment.rb / initializer)
  def request_authentication
    unless authenticated?
      respond_to do |format|
        format.xml { request_http_basic_authentication('Path Creation Tool') }
        format.html { redirect_to path_with_referer(login_url) }
      end
    end 
  end

  def authenticated?
    !@authenticated_user.nil?
  end
  
  def authorized?(authorized_user_names, allow_admin = nil)
    allow_admin = true if allow_admin.nil?
    authenticated? && ((allow_admin && @authenticated_user.admin?) || authorized_user_names.include?(@authenticated_user.name))
  end
  
  def with_authorization(*args)
    options = args.extract_options!
    if (options.has_key?(:if) && options[:if] == false) || (options.has_key?(:unless) && options[:unless] == true)
      block_given? ? yield : return
    else
      authorized_user_names = args.shift || []
      with_authentication do
        authorized?(authorized_user_names, options[:admin]) ? (block_given? ? yield : return) : http_forbidden
      end
    end
  end
  
  def with_authentication(*args)
    options = args.extract_options!
    if (options.has_key?(:if) && options[:if] == false) || (options.has_key?(:unless) && options[:unless] == true)
      block_given? ? yield : return
    else
      authenticated? ? (block_given? ? yield : return) : request_authentication
    end
  end
  
  def empty_xml_document(root)
    unless root.is_a? String
      raise ArgumentError, "Expected String, but got #{root.class}"
    end
    doc = REXML::Document.new 
    doc.add_element root
    doc << REXML::XMLDecl.new('1.0', 'UTF-8')
  end
  
  def atom_for_path(path)
    feed = Atom::Feed.new
    feed.title = path.name
    feed.subtitle = path.description
    most_recent_point = path.points.max do |p1, p2|
      p1.updated_at <=> p2.updated_at
    end
    feed.updated = most_recent_point.updated_at
    
    # The path's owning user is the feed's author
    feed.authors << user_to_atom_author(path.user)
    
    # Add each point as an entry in the feed
    path.points.find(:all, :order => 'position').each do |point|
      entry = feed.entries.new
      entry.title = point.name
      entry.content = point.description
      entry.updated = point.updated_at
      
      # Link to the external resource
      external_uri = entry.links.new
      external_uri['href'] = point.uri
    end  
    feed.to_xml
  end
  
  def atom_for_path_list(paths, user = nil)
    feed = Atom::Feed.new
    feed.title = "Collection of paths"
    #feed.subtitle = path.description
    most_recent_path = paths.max do |p1, p2|
      p1.updated_at <=> p2.updated_at
    end
    feed.updated = most_recent_path.updated_at
    
    # If a user is specified, it is the feed's author
    if user
      feed.authors << user_to_atom_author(user)
    end
    
    # Add each path as an entry in the feed
    paths.each do |path|
      entry = feed.entries.new
      entry.title = path.name
      entry.content = path.description
      entry.updated = path.updated_at
      
      unless user
        entry.authors << user_to_atom_author(path.user)
      end
      
      # Link to the external resource
      external_uri = entry.links.new
      external_uri['href'] = user_path_url(path.user.name, path.name)
    end
    
    feed.to_xml
  end
  
  def user_to_atom_author(user)
    Atom::Author.new(:name => user.full_name, :uri => user_url(user.name))
  end
  
  def find_categories
    @categories = Category.find(:all, :order => 'name')
  end
  
  def render_paths_as_xml(paths)
    if paths.blank?
      render :xml => empty_xml_document('paths')
    else
      render :xml => paths.to_xml(:only => Path.xml_fields, :include => [:user])
    end
  end
  
  def path_with_referer(path, referer = nil)
    referer ||= (self.relative_url_root + self.request.path)
    path << ( path.include?('?') ? '&' : '?' ) + referer.to_query('referer')
  end
end
