# TODO: Incorporate categories into Atom output
class PathsController < ApplicationController
  include PCT::ResourceController
  
  # All requests for path resources need a valid {user-name} in the URI
  before_filter :find_user, :except => :viewer
  
  # Look for path in db
  before_filter :find_path, :except => [ :index, :new, :create, :viewer ]
    
  before_filter :find_categories, :except => :viewer
  
  # GET /users/{user-name}/paths
  # TODO: Include link to Atom in HTML
  def index
    # Privacy. Only return public paths unless authenticated as owning user.
    if authorized?(@user.name)
      @paths = Path.paginate_by_user_id(@user.id, :page => params[:page], :order => 'name ASC')
    else
      @paths = Path.paginate_by_user_id_and_public(@user.id, true, :page => params[:page], :order => 'name ASC')
    end
    
    respond_to do |format|
      format.html
      format.xml { render_paths_as_xml @paths }
      format.atom { render :xml => atom_for_path_list(@paths), :content_type => 'application/atom+xml' }
    end
  end
  
  # GET /users/{user-name}/paths/new
  # Form for new path creation
  def new
    @path = Path.new
  end

  # POST /users/{user-name}/paths
  def create
    with_authorization(@user.name, :admin => false) do
      @path = Path.new do |p|
        p.user_id = @user.id

        if params[:path]
          if categories = params[:path].delete(:categories)
            categories.each do |cat|
              if category = Category.find_by_name(cat)
                p.categories << category
              end
            end
          end
          
          p.attributes = params[:path]
        end
      end
      
      save_resource( @path, [ @user, @path ], :xml_only => Path.xml_fields, :xml_include => Path.xml_include )
    end
  end
  
  # GET /users/{user-name}/paths/{path-alias}
  # TODO: Include link to Atom in HTML
  def show
    # Require authorization unless path is public.
    with_authorization(@user.name, :unless => @path.public?) do
      respond_to do |format|
        format.html
        format.xml { render :xml => @path.to_xml(:only => Path.xml_fields, :include => Path.xml_include) }
        format.atom { render :xml => atom_for_path(@path), :content_type => 'application/atom+xml' }
      end
    end
  end
  
  # PUT /users/{user-name}/paths/{path-alias}
  def update
    # Update path if it already exists, if authenticated as owning user
    with_authorization(@user.name) do
      unless params[:path].nil?
        @path.categories = []
        if categories = params[:path].delete(:categories)
          categories.each do |cat|
            if category = Category.find_by_name(cat)
              @path.categories << category
            end
          end
        end
        
        @path.attributes = params[:path]
      end
    
      save_resource( @path, [ @user, @path ], :xml_only => Path.xml_fields, :xml_include => Path.xml_include )
    end
  end
  
  # DELETE /users/{user-name}/paths/{path-alias}
  def destroy
    # Only authenticated user can delete their own paths
    with_authorization(@user.name) do
      respond_to do |format|
        format.xml do
          @path.destroy ? http_ok : http_internal_server_error
        end
        format.html do
          if @path.destroy
            flash[:notice] = "Path deleted."
            redirect_to @user
          else
            flash[:warning] = "Failed to delete path."
            render :template => 'delete'
          end
        end
      end
    end
  end
  
  def viewer
    unless @path = Path.find_by_alias(params[:alias], :include => :points)
      http_not_found
      return
    end
    @user = @path.user
    
#    position = (params[:position] || 1).to_i
#    index = nil
#    @path.points.each_index do |i|
#      if @path.points[i].position == position
#        index = i
#        break
#      end
#    end
    
#    if index.nil?
#      logger.debug("Position #{position} not present in path's points.")
#      http_not_found
#      return
#    end
    
    page = params[:page].to_i
    unless page.between?(1, @path.points.size)
      http_not_found
      return
    end
  
    respond_to do |format|
      format.html do
        @points = @path.points.paginate(:per_page => 1, :page => page, :order => 'position ASC')
        @point = @points.first
        @path_viewer = true
      end
    end
  end
  
  private
  def find_user
    @user = User.find_by_name(params[:user_id])
    unless @user
      logger.debug("User not found, ID: #{params[:user_id]}")
      http_not_found
    end
  end
  
  def find_path
    @path = Path.find_by_user_id_and_alias(@user.id, params[:id])
    unless @path
      logger.debug("Path not found: user id #{@user.id}, alias #{params[:id]}")
      http_not_found
    end
  end
end
