class PointsController < ApplicationController
  include PCT::ResourceController
  
  # All requests for path resources need a valid {user-name} in the URI
  before_filter :find_user
  
  # All requests for path resources need a valid {path-alias} in the URI
  before_filter :find_path
  
  # Look for point in db
  before_filter :find_point, :except => [ :index, :create, :new ]
  
  # GET /users/{user-name}/paths/{path-alias}/points
  def index
    @points = Point.find_all_by_path_id(@path.id, :order => 'position')
    respond_to do |format|
      format.html
      format.xml { render :xml => (@points.blank? ? empty_xml_document('points').to_s : @points.to_xml(:only => Point.xml_fields)) }
    end
  end
  
  # GET /users/{user-name}/paths/{path-alias}/points/new
  # Form for new point creation
  def new
    @point = Point.new
  end
  
  # POST /users/{user-name}/paths/{path-alias}/points
  def create
    with_authorization(@user.name, :admin => false) do
      @point = Point.new do |p|
        p.path_id = @path.id
        unless params[:point].nil?
          p.attributes = params[:point]
        end
      end

      save_resource( @point, [ @user, @path, @point ], :xml_only => Point.xml_fields )
    end
  end
  
  # GET /users/{user-name}/paths/{path-alias}/points/{point-position}
  def show
    respond_to do |format|
      format.html
      format.xml { render :xml => @point.to_xml(:only => Point.xml_fields) }
    end
  end
  
  # GET /users/{user-name}/paths/{path-alias}/points/{position}/edit
  # Form for editing point
  def edit
    # Only authenticated user can edit their paths
    with_authorization(@user.name) do
      respond_to do |format|
        format.html
      end
    end
  end
  
  # PUT /users/{user-name}/paths/{path-alias}/points/{position}
  def update
    # Update point if it already exists, if authenticated as owning user
    with_authorization(@user.name) do
      unless params[:point].nil?
        @point.attributes = params[:point]
      end
      
      position_changed = @point.position_changed?

      save_resource( @point, [ @user, @path, @point ], :xml_only => Point.xml_fields )
    end
  end
  
  # DELETE /users/{user-name}/paths/{path-alias}/points/{position}
  def destroy
    # Only authenticated user can delete points from their paths
    with_authorization(@user.name) do
      respond_to do |format|
        format.xml do
          @point.destroy ? http_ok : http_internal_server_error
        end
        format.html do
          if @point.destroy
            flash[:notice] = "Point deleted."
            redirect_to [ @user, @path ]
          else
            flash[:warning] = "Failed to delete point."
            render :template => 'delete'
          end
        end
      end
    end
  end

  private
    
  def find_user
    unless @user = User.find_by_name(params[:user_id])
      http_not_found
    end
  end
  
  def find_path
    unless @path  = Path.find_by_user_id_and_alias(@user.id, params[:path_id])
      http_not_found
      return
    end
    
    # Points from private paths require authorisation as owner of path
    with_authorization(@path.user.name, :unless => @path.public?)
  end
  
  def find_point
    unless @point = Point.find_by_path_id_and_position(@path.id, params[:id])
      http_not_found
    end
  end
end
