# TODO: Require confirmation before allowing a user to stop themselves being an admin
class UsersController < ApplicationController
  include PCT::ResourceController
  
  before_filter :protect_admin_status!, :only => [ :update, :create ]
  before_filter :find_user, :except => [ :index, :new, :create, :update ]
    
  after_create :login_new_user
  
  # GET /users
  def index
    respond_to do |format|
      format.html do
        @users = User.paginate(:page => params[:page], :order => 'name ASC')
      end
    end
  end
  
  # GET /users/new
  def new
    @user = User.new
    
    # TODO: Respond to GET /users/new.xml with XML description of new user.
    #       Below is not sufficient as omits password property & includes updated_at.
    #respond_to do |format|
      #format.xml { render :xml => @user.to_xml }
    #end
  end
  
  # POST /users
  # Or PUT /users/{user-name} for non-existent user name,
  # called from update method.
  def create
    @user = User.new do |u|
      if params[:user]
        u.attributes = params[:user]
      end
      
      # Specifying the user name in the URL takes precedence
      if params[:id]
        u.name = params[:id]
      end
      
      # First user account created is an admin account
      if User.count == 0
        u.admin = true
      end
    end

    save_resource( @user, @user, :xml_only => User.private_fields, :conflict => User.default_error_messages[:unique_name] )
  end
    
  # GET /users/{user-name}
  def show
    respond_to do |format|
      format.xml do
        # Privacy: only output user name unless authenticated as user.
        render :xml => @user.to_xml(:only => authorized?(@user.name) ? User.private_fields : User.public_fields)
      end
      format.html do
        conditions = authorized?(@user.name) ? [] : [ 'public = ?', true ]
        @paths = Path.paginate_by_user_id(@user.id, :page => params[:page], :order => 'name ASC', :conditions => conditions)
      end
    end
  end
  
  # GET /users/{user-name}/edit
  def edit
    # Only authenticated user can edit their own account
    with_authorization(@user.name) do
      @user[:password] = ''
      @user[:email_confirmation] = @user[:email]
    end
  end
  
  # PUT /users/{user-name}
  def update
    # Delegate to create method for new account
    unless @user = User.find_by_name(params[:id])
      create
      return
    end
    
    # Interpret unauthenticated request as an attempt to create a new
    # account with a taken name
    unless authenticated?
      create
      return
    end
    
    # Update account if it already exists, if authenticated as that user
    with_authorization(@user.name) do
      unless params[:user].nil?
        # On update, interpret blank password value as unchanged password
        # TODO: Write functional test for this
        if params[:user].include?(:password) && params[:user][:password].blank?
          params[:user].delete(:password)
        end
        @user.attributes = params[:user]
      end
    
      save_resource( @user, @user, :xml_only => User.private_fields, :conflict => User.default_error_messages[:unique_name] )
    end
  end
  
  # TODO: Should send description of object deleted if status is 200, as per RFC2616 <http://www.w3.org/Protocols/rfc2616/rfc2616-sec9.html#sec9.7>
  # DELETE /users/{user-name}
  def destroy
    # Only authenticated user can delete their own account
    with_authorization(@user.name) do
      respond_to do |format|
        format.xml do
          @user.destroy ? http_ok : http_internal_server_error
        end
        format.html do
          if @user.destroy
            if session[:authenticated_user_id] == @user.id
              session.delete(:authenticated_user_id)
            end
            flash[:notice] = "Account deleted."
            redirect_to root_url
          else
            flash[:warning] = "Failed to delete account."
            render :template => 'delete'
          end
        end
      end
    end
  end  
  
  private
  def login_new_user
    if request.format == Mime::HTML
      session[:authenticated_user_id] = @user.id unless authenticated?
    end
  end
  
  def protect_admin_status!
    if params[:user] && params[:user].include?(:admin)
      unless authenticated? && @authenticated_user.admin?
        # TODO: Make error message configurable, as per model default error messages
        http_forbidden('Admin status can only be set by existing admins.')
      end
    end
  end
  
  def find_user
    @user = User.find_by_name(params[:id])
    unless @user
      logger.debug("User not found, ID: #{params[:id]}")
      http_not_found
    end
  end
end
