class PasswordResetKey < ActiveRecord::Base
  # Possible characters for use in random reset key 
  RANDOM_CHARSET = (('A'..'Z').to_a + ('0'..'9').to_a)
  
  # Duration of validity of reset key, in seconds
  TTL = 60 * 60
  
  cattr_reader :default_error_messages
  @@default_error_messages = {
    :user_id_unique => 'already has an active password reset key, which will expire one hour after creation',
    :user_required => 'account not found'
  }
  
  validates_presence_of :user, :message => @@default_error_messages[:user_required]
  validates_presence_of :key
  validates_uniqueness_of :user_id, :message => @@default_error_messages[:user_id_unique]
  
  before_validation_on_create :randomize_key!
  before_save :auto_expiry!
  
  belongs_to :user
  
  def to_param
    self.key
  end
  
  def expired?
    !new_record? && (Time.zone.now > self.expiry)
  end
  
  private
  
  # Sets a unique random key
  def randomize_key!
    while self.key.nil? || self.class.find_by_key(self.key)
      self.key = self.class.random
    end
  end
  
  # Generates a random key as a 4x4 string, like 6QIG-H6O4-Q5LP-SV1P
  def self.random
    random_key_chunks = []
    4.times do |i|
      chunk = ''
      4.times do |j|
        chunk += RANDOM_CHARSET[rand(RANDOM_CHARSET.size)]
      end
      random_key_chunks.push(chunk)
    end
    random_key_chunks.join('-')
  end
  
  def auto_expiry!
    self.expiry = Time.zone.now + TTL
  end  
end
