# TODO: Prevent alias being changed post-creation
class Path < ActiveRecord::Base
  cattr_reader :xml_fields, :xml_include, :default_error_messages
  
  @@xml_fields = [ :name, :description, :public, :alias ]
  
  @@xml_include = [ :categories, :user ]
  
  @@default_error_messages = {
    :control_characters => 'can not contain control characters',
    :parent_blank => 'can not be blank and must already exist',
    :boolean => 'must be true or false',
    :constant_user_id => 'can not be changed',
    :reserved_name => 'is a reserved name',
    :unique_name => 'you already have a path with this name'
  }
  
  # Names that should not be permitted, to prevent URI namespace clashes
  RESERVED_NAMES = %w( new )
  
  # Prevent mass-assignment of other attributes
  attr_accessible :user_id, :name, :description, :public
  
  # User can not be changed after path creation
  attr_readonly :user_id
  
  # Path is a child of User
  belongs_to :user
  
  # Path is the parent of Points
  has_many :points, :dependent => :destroy, :order => 'position ASC, name ASC'
  
  # A path can be associated with many categories
  has_and_belongs_to_many :categories, :join_table => 'paths_categories', :order => 'name ASC'
  
  validate :validate_no_control_characters_in_name
  validates_presence_of :name
  validates_length_of :name, :maximum => 100
  validates_presence_of :user_id, :message => @@default_error_messages[:parent_blank]
  validates_inclusion_of :public, :in => [true, false], :message => @@default_error_messages[:boolean]
  validates_exclusion_of :name, :in => RESERVED_NAMES, :message => @@default_error_messages[:reserved_name]
  validates_uniqueness_of :name, :scope => :user_id, :if => :user_id?, :message => @@default_error_messages[:unique_name]
  validates_uniqueness_of :alias, :allow_nil => true
  validate_on_update :validate_constant_user_id
  
  before_create :unique_alias!
  
  def to_param
    self.alias_was
  end
  
  # Gets the next available position in the path
  def next_position
    Point.maximum('position', :conditions => ['path_id=?', self.id]).to_i + 1
  end
  
  # Adds an error to name if it contains ASCII control characters
  def validate_no_control_characters_in_name
    if name && name.match(/[\x00-\x1f\x7f-\x9f]/)
      errors.add(:name, @@default_error_messages[:control_characters])
    end
  end
  
  # Adds an error to user_id if it was changed on a saved path
  def validate_constant_user_id
    if user_id_changed?
      errors.add(:user_id, @@default_error_messages[:constant_user_id])
    end
  end
  
  # Generate a random, unique 6 character alias
  def self.unique_alias
    random_charset = (('a'..'z').to_a + ('0'..'9').to_a)
    ualias = ''
    while ualias.blank? || self.find_by_alias(ualias)
      ualias = ''
      6.times do |j|
        ualias += random_charset[rand(random_charset.size)]
      end      
    end
    ualias
  end
  
  # Set this path's alias to a random, unique 6 character string
  def unique_alias!
    self.alias = self.class.unique_alias
  end
end
