# TODO: Make first user created an admin
class User < ActiveRecord::Base
  include BCrypt
  
  attr_accessor :password
  
  cattr_reader :public_fields, :private_fields, :default_error_messages

  @@public_fields = [ :name ]
  
  @@private_fields = [ :email, :full_name, :name, :admin ]

  @@default_error_messages = {
    :name_valid_characters => 'can only contain lower-case letters (a-z) and numbers (0-9)',
    :name_first_character => 'must start with a lower-case letter (a-z)',
    :name_last_character => 'must end with a lower-case letter (a-z) or number (0-9)',
    :common_password => 'is commonly used and insecure',
    :reserved_name => 'is a reserved name',
    :password_equals_name => 'should not be the same as the user name',
    :preserve_last_admin => 'can not be false because this is the last admin account',
    :email_confirmation => 'must match confirmation',
    :password_confirmation => 'must match confirmation',
    :unique_name => 'has already been registered',
    :unique_email => 'has already been registered',
    :email_format => 'is not a valid email address'
  }

  # Possible characters for use in random password 
  RANDOM_CHARSET = (('A'..'Z').to_a + ('0'..'9').to_a)
  
  # Names that should not be permitted, to prevent URI namespace clashes
  RESERVED_NAMES = %w( new )
  
  # Prevent mass-assignment of other attributes
  attr_accessible :name, :full_name, :email, :email_confirmation, :password, :password_confirmation, :admin
  
  # User is the parent of Paths
  has_many :paths, :dependent => :destroy
  
  has_one :password_reset_key, :dependent => :destroy
  
  validates_presence_of :name, :full_name, :email, :password
  validates_uniqueness_of :name, :message => @@default_error_messages[:unique_name]
  validates_uniqueness_of :email, :message => @@default_error_messages[:unique_email]
  validates_length_of :name, :in => 3..32
  validates_format_of :name, :with => /^[a-z0-9]*$/, :message => @@default_error_messages[:name_valid_characters]
  validates_format_of :name, :with => /^[a-z]/, :message => @@default_error_messages[:name_first_character]
  validates_format_of :name, :with => /[a-z0-9]$/, :message => @@default_error_messages[:name_last_character]
  validates_format_of :email, :with => /^[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+\.[a-zA-Z]{2,4}$/, :message => @@default_error_messages[:email_format]
  validates_exclusion_of :name, :in => RESERVED_NAMES, :message => @@default_error_messages[:reserved_name]
  validate :validate_preserve_last_admin
  validate :validate_common_passwords, :if => :password_hash_changed?
  validate :validate_name_password_different, :if => :password_hash_changed?
  validates_length_of :password, :in => 6..32, :if => :password_hash_changed?
  validates_confirmation_of :email, :message => @@default_error_messages[:email_confirmation]
  validates_confirmation_of :password, :message => @@default_error_messages[:password_confirmation], :if => :password_hash_changed?
  
  before_validation :plain_password!
  after_validation :obscure_password!
  
  def self.default_error_messages
    @@default_error_messages
  end
  
  def self.random_password
    random_pw = ''
    8.times do |i|
      random_pw += RANDOM_CHARSET[rand(RANDOM_CHARSET.size)]
    end
    random_pw
  end
  
  def to_param
    self.name_was
  end
  
  def password
    if @password.nil? && !password_hash.blank? && !password_hash_changed?
      @password ||= Password.new(password_hash)
    end
    @password
  end

  def password=(new_password)
    @new_password = new_password # Retained for validation; not stored in DB.
    @password = Password.create(new_password)
    self.password_hash = @password
  end
  
  # Authenticates a user name and password against the database
  # Returns the user if details are valid, otherwise FALSE.
  def self.authenticate(name, password)
    (user = find_by_name(name)) && (user.password == password) ? user : false
  end
  
  private
  # Do not allow common passwords
  def validate_common_passwords
    if COMMON_PASSWORDS.include?(password)
      errors.add(:password, @@default_error_messages[:common_password])
    end
  end
  
  # Password should not be the same as the user name
  def validate_name_password_different
    if password == name
      errors.add(:password, @@default_error_messages[:password_equals_name])
    end
  end
  
  # Always keep at least one admin account
  def validate_preserve_last_admin
    if (admin_was == true) && (admin == false)
      unless User.count( :conditions => [ 'admin = ? AND id <> ?', true, self.id ] ) > 0
        errors.add(:admin, @@default_error_messages[:preserve_last_admin])
      end
    end
  end
  
  # Switch the password to its plain value for validation purposes
  def plain_password!
    @password = @new_password
  end
    
  # Switch the password back to its hash
  def obscure_password!
    @password = self.password_hash
  end
end
