require 'test_helper'
require 'atom/feed'

class PathsControllerTest < ActionController::TestCase

  # Check that index without authorization gets only public paths
  test "index_200_public" do
    user = users(:bob)
    get :index, :user_id => user.name, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'paths', ['path'])
    user.paths.each do |path|
      assert path.public? == assigns(:paths).include?(path)
    end
  end
    
  # Check that index with authorization gets public and private paths
  test "index_200_private" do
    user = users(:bob)
    http_auth_as(user.name)
    get :index, :user_id => user.name, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'paths', ['path'])
    user.paths.each do |path|
      assert assigns(:paths).include?(path)
    end
  end
  
  # Check that index on user with no public paths return empty <paths/>
  test "index_200_empty" do
    user = users(:admin)
    get :index, :user_id => user.name, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'paths', [])
  end
  
  # Check that index on non-existent user name gets 404
  test "index_404" do
    get :index, :user_id => 'Non-existent user'
    assert_response_code 404
  end
    
  # Check that requesting a public path without auth gets 200
  test "show_200_public" do
    path = paths(:wilf_poems)
    get :show, :user_id => path.user.name, :id => path.alias, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'path', ['description', 'public', 'name', 'categories', 'user', 'alias'])
  end

  # Check that requesting a private path with authorization gets 200
  test "show_200_private" do
    path = paths(:search_engines_private)
    http_auth_as(path.user.name)
    get :show, :user_id => path.user.name, :id => path.alias, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'path', ['description', 'public', 'name', 'categories', 'user', 'alias'])
  end
  
  # Check that requesting a path as Atom gets valid Atom feed
  test "atom_path" do
    path = paths(:wilf_poems)
    get :show, :user_id => path.user.name, :id => path.alias, :format => 'atom'
    assert_response_code 200
    assert_nothing_raised do 
      Atom::Feed.parse(@response.body).inspect
    end
  end
  
  # Check that requesting a private path without auth gets 401
  test "show_401" do
    path = paths(:search_engines_private)
    get :show, :user_id => path.user.name, :id => path.alias
    assert_response_code 401
  end
    
  # Check that requesting a private path as other user gets 403
  test "show_403" do
    path = paths(:search_engines_private)
    http_auth_as(users(:jane).name)
    get :show, :user_id => path.user.name, :id => path.alias, :format => 'xml'
    assert_response_code 403
  end
  
  # Check that requesting a non-existent path gets 404
  test "show_404" do
    user = users(:bob)
    get :show, :user_id => user.name, :path_id => 'Non-existent path'
    assert_response_code 404
  end
  
  # Check that POST to /users/{user-name}/paths with authorization gets 201,
  # and created a new resource
  test "create_201" do
    user = users(:bob)
    path_name = 'New path'
    http_auth_as(user.name)
    post :create, :user_id => user.name, :path => {:name => path_name, :public => true}, :format => 'xml'
    assert_response_code 201
    assert_valid_xml_response(@response, 'path', ['description', 'public', 'name', 'categories', 'user', 'alias'])
    assert_equal user_path_url(user.name, assigns(:path).alias), @response.headers['Location']
    assert Path.find_by_user_id_and_name(user.id, path_name)
  end
  
  # Check that POST to /users/{user-name}/paths fails to create resource with 400 if authenticated but bad request data
  test "create_400" do
    user = users(:bob)
    path_name = 'New path'
    http_auth_as(user.name)
    post :create, :user_id => user.name, :path => {:name => path_name} # incomplete data
    assert_response_code 400
    assert !Path.find_by_user_id_and_name(user.id, path_name)
  end

  # Check that POST to /users/{user-name}/paths fails to create resource with 401 if unauthenticated
  test "create_401" do
    user = users(:bob)
    path_name = 'New path'
    post :create, :user_id => user.name, :path => {:name => path_name, :public => true}
    assert_response_code 401
    assert !Path.find_by_user_id_and_name(user.id, path_name)
  end
  
  # Check that POST to /users/{user-name}/paths fails to create resource with 403 if authenticated but unauthorized
  test "create_403" do
    user = users(:bob)
    path_name = 'New path'
    http_auth_as(users(:jane).name)
    post :create, :user_id => user.name, :path => {:name => path_name, :public => true}
    assert_response_code 403
    assert !Path.find_by_user_id_and_name(user.id, path_name)
  end

  # Check that POST to /users/{user-name}/paths fails to create resource with 403 if authenticated but unauthorized, even if admin
  test "admin_create_403" do
    user = users(:bob)
    path_name = 'New path'
    http_auth_as(users(:admin).name)
    post :create, :user_id => user.name, :path => {:name => path_name, :public => true}
    assert_response_code 403
    assert !Path.find_by_user_id_and_name(user.id, path_name), "Admin users should not be able to create paths as other users"
  end
  
  # Check that PUT to existing path updates resource with 200 if authorized
  test "update_200" do
    user = users(:bob)
    path_alias = user.paths.first.alias
    http_auth_as(user.name)
    put :update, :user_id => user.name, :id => path_alias, :path => {:description => 'New description'}
    assert_response_code 200
    assert_valid_xml_response(@response, 'path', ['description', 'public', 'name', 'categories', 'user', 'alias'])
    assert_equal 'New description', Path.find_by_user_id_and_alias(user.id, path_alias).description
  end

  # Check that PUT to existing path name fails to update resource with 400 if authenticated but bad request data
  test "update_400" do
    user = users(:bob)
    path_alias = user.paths.first.alias
    http_auth_as(user.name)
    put :update, :user_id => user.name, :id => path_alias, :path => {:public => nil}
    assert_response_code 400
  end
  
  # Check that PUT changing user name fails
  test "update_400_user_name" do
    user = users(:bob)
    path_alias = user.paths.first.alias
    http_auth_as(user.name)
    put :update, :user_id => user.name, :id => path_alias, :path => {:user_id => users(:jane).id}
    assert_response_code 400
  end
  
  # Check that PUT to existing path name fails to update resource with 401 if unauthenticated
  test "update_401" do
    user = users(:bob)
    path_alias = user.paths.first.alias
    put :update, :user_id => user.name, :id => path_alias, :path => {:description => 'New description'}
    assert_response_code 401
    assert_not_equal 'New description', Path.find_by_user_id_and_alias(user.id, path_alias).description
  end
  
  # Check that PUT to existing path name fails to update resource with 403 if authenticated but unauthorized
  test "update_403" do
    user = users(:bob)
    path_alias = user.paths.first.alias
    http_auth_as(users(:jane).name)
    put :update, :user_id => user.name, :id => path_alias, :path => {:description => 'New description'}
    assert_response_code 403
    assert_not_equal 'New description', Path.find_by_user_id_and_alias(user.id, path_alias).description
  end
  
  # Check that DELETE to non-existent path fails with 404
  test "destroy_404" do
    user = users(:bob)
    delete :destroy, :user_id => user.name, :id => 'Non-existent path'
    assert_response_code 404
  end
      
  # Check that DELETE succeeds with 200 if authorized
  test "destroy_200" do
    user = users(:bob)
    path_alias = user.paths.first.alias
    http_auth_as(user.name)
    delete :destroy, :user_id => user.name, :id => path_alias
    assert_response_code 200
    assert !Path.find_by_user_id_and_alias(user.id, path_alias)
    delete :destroy, :user_id => user.name, :id => path_alias
    assert_response_code 404
  end
  
  # Check that DELETE fails with 401 if unauthenticated
  test "destroy_401" do
    user = users(:bob)
    path_alias = user.paths.first.alias
    delete :destroy, :user_id => user.name, :id => path_alias
    assert_response_code 401
    assert Path.find_by_user_id_and_alias(user.id, path_alias)
  end
  
  # Check that DELETE fails with 403 if authenticated but unauthorized
  test "destroy_403" do
    user = users(:bob)
    path_alias = user.paths.first.alias
    http_auth_as(users(:jane).name)
    delete :destroy, :user_id => user.name, :id => path_alias
    assert_response_code 403
    assert Path.find_by_user_id_and_alias(user.id, path_alias)
  end
end
