require 'test_helper'

class PointsControllerTest < ActionController::TestCase
  # Check that GET index on public path with points gets 200
  test "index_200_public" do
    user = users(:bob)
    path = paths(:wilf_poems)
    get :index, :user_id => user.name, :path_id => path.alias, :format => 'xml'
    assert_response_code 200
    xml = assert_valid_xml_response(@response, 'points', ['point'])
    assert_equal path.points.size, REXML::XPath.match(xml, '//points/point').size
  end
  
  # Check that GET index on empty public path gets 200
  test "index_200_empty" do
    user = users(:bob)
    path = paths(:empty_path)
    get :index, :user_id => user.name, :path_id => path.alias, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'points', [])
  end
  
  # Check that GET index on private path with authorization gets 200
  test "index_200_private" do
    user = users(:bob)
    path = paths(:search_engines_private)
    http_auth_as(user.name)
    get :index, :user_id => user.name, :path_id => path.alias, :format => 'xml'
    assert_response_code 200
    xml = assert_valid_xml_response(@response, 'points', ['point'])
    assert_equal path.points.size, REXML::XPath.match(xml, '//points/point').size
  end
  
  # Check that GET index on private path without authentication gets 401
  test "index_401_private" do
    user = users(:bob)
    path = paths(:search_engines_private)
    get :index, :user_id => user.name, :path_id => path.alias
    assert_response_code 401
  end
  
  # Check that GET index on private path without authorization gets 403
  test "index_403_private" do
    user = users(:bob)
    path = paths(:search_engines_private)
    http_auth_as(users(:jane).name)
    get :index, :user_id => user.name, :path_id => path.alias
    assert_response_code 403
  end
  
  # Check that GET show on public path's point gets 200
  test "show_200_public" do
    point = points(:wilf_anthem)
    get :show, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'point', ['description', 'position', 'name', 'uri'])
  end
  
  # Check that GET show on private path's point with authorization gets 200
  test "show_200_private" do
    user = users(:bob)
    path = paths(:search_engines_private)
    http_auth_as(user.name)
    get :show, :user_id => user.name, :path_id => path.alias, :id => 1, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'point', ['description', 'position', 'name', 'uri'])
  end
  
  # Check that GET show on private path's point without authentication gets 401
  test "show_401_private" do
    user = users(:bob)
    path = paths(:search_engines_private)
    get :show, :user_id => user.name, :path_id => path.alias, :id => 1
    assert_response_code 401
  end
  
  # Check that GET show on private path's point without authorization gets 403
  test "show_403_private" do
    user = users(:bob)
    path = paths(:search_engines_private)
    http_auth_as(users(:jane).name)
    get :show, :user_id => user.name, :path_id => path.alias, :id => 1
    assert_response_code 403
  end
  
  # Check that GET show on non-existant point gets 404
  test "show_404" do
    path = paths(:wilf_poems)
    get :show, :user_id => path.user.name, :path_id => path.alias, :id => path.next_position
    assert_response_code 404
  end
  
  # Check that POST with authorization creates point and gets 201
  test "create_201" do
    path = paths(:wilf_poems)
    next_position = path.next_position
    http_auth_as(path.user.name)
    post :create, :user_id => path.user.name, :path_id => path.alias, :point => {:uri => 'http://www.example.com/', :name => 'New point'}
    assert_response_code 201
    assert_valid_xml_response(@response, 'point', ['description', 'position', 'name', 'uri'])
    assert_equal user_path_point_url(path.user.name, path.alias, next_position), @response.headers['Location']
    assert Point.find_by_path_id_and_position(path.id, next_position)
  end
  
  # Check that POST with bad data gets 400
  test "create_400" do
    path = paths(:wilf_poems)
    next_position = path.next_position
    http_auth_as(path.user.name)
    post :create, :user_id => path.user.name, :path_id => path.alias
    assert_response_code 400
    assert !Point.find_by_path_id_and_position(path.id, next_position)
  end
  
  # Check that POST without authentication gets 401
  test "create_401" do
    path = paths(:wilf_poems)
    next_position = path.next_position
    post :create, :user_id => path.user.name, :path_id => path.alias, :point => {:uri => 'http://www.example.com/', :name => 'New point'}
    assert_response_code 401
    assert !Point.find_by_path_id_and_position(path.id, next_position)
  end
  
  # Check that POST without authorization gets 403
  test "create_403" do
    path = paths(:wilf_poems)
    next_position = path.next_position
    http_auth_as(users(:jane).name)
    post :create, :user_id => path.user.name, :path_id => path.alias, :point => {:uri => 'http://www.example.com/', :name => 'New point'}
    assert_response_code 403
    assert !Point.find_by_path_id_and_position(path.id, next_position)
  end

  # Check that POST without authorization gets 403, even as admin
  test "admin_create_403" do
    path = paths(:wilf_poems)
    next_position = path.next_position
    http_auth_as(users(:admin).name)
    post :create, :user_id => path.user.name, :path_id => path.alias, :point => {:uri => 'http://www.example.com/', :name => 'New point'}
    assert_response_code 403
    assert !Point.find_by_path_id_and_position(path.id, next_position), "Admin users should not be able to create points as other users"
  end

  # Check that PUT with authorization updates point and gets 200
  test "update_200" do
    point = points(:wilf_dulce)
    http_auth_as(point.path.user.name)
    put :update, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position, :point => {:name => 'New name'}
    assert_response_code 200
    assert_valid_xml_response(@response, 'point', ['description', 'position', 'name', 'uri'])
    assert_equal 'New name', Point.find(point.id).name
  end
  
  # Check that changing the position gets 301 with new URL
  test "update_301" do
    point = paths(:wilf_poems).points.last
    new_position = point.position + 1
    http_auth_as(point.path.user.name)
    put :update, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position, :point => {:position => new_position}, :format => 'xml'
    assert_response_code 301
    assert_equal user_path_point_url(point.path.user.name, point.path.alias, new_position), @response.headers['Location']
  end
  
  # Check that PUT with bad data gets 400
  test "update_400" do
    point = points(:wilf_dulce)
    http_auth_as(point.path.user.name)
    put :update, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position, :point => {:name => ''}
    assert_response_code 400
    assert_not_equal '', Point.find(point.id).name
  end
  
  # Check that PUT without authentication gets 401
  test "update_401" do
    point = points(:wilf_dulce)
    put :update, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position, :point => {:name => 'New name'}
    assert_response_code 401
    assert_not_equal 'New name', Point.find(point.id).name
  end
  
  # Check that PUT without authorization gets 403
  test "update_403" do
    point = points(:wilf_dulce)
    http_auth_as(users(:jane).name)
    put :update, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position, :point => {:name => 'New name'}
    assert_response_code 403
    assert_not_equal 'New name', Point.find(point.id).name
  end
  
  # Check that PUT to non-existent point gets 404
  test "update_404" do
    point = points(:wilf_dulce)
    http_auth_as(point.path.user.name)
    put :update, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.path.next_position, :point => {:name => 'New name'}
    assert_response_code 404
  end
  
  # Check that POST to points/1 gets 405 (method not allowed)
  # test "update_405" do
  #   point = points(:wilf_dulce)
  #   http_auth_as(point.path.user.name)
  #   post :update, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position, :point => {:name => 'New point'}
  #   assert_response_code 405
  #   assert_not_equal Point.find_by_path_id_and_position(path.id, 1).name, 'New point'
  # end
  
  # Check that DELETE with authorization updates point and gets 200
  test "destroy_200" do
    point = points(:wilf_dulce)
    http_auth_as(point.path.user.name)
    delete :destroy, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position
    assert_response_code 200
    assert !Point.find_by_id(point.id)
  end
    
  # Check that DELETE without authentication gets 401
  test "destroy_401" do
    point = points(:wilf_dulce)
    delete :destroy, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position
    assert_response_code 401
    assert Point.find_by_id(point.id)
  end
  
  # Check that DELETE without authorization gets 403
  test "destroy_403" do
    point = points(:wilf_dulce)
    http_auth_as(users(:jane).name)
    delete :destroy, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.position
    assert_response_code 403
    assert Point.find_by_id(point.id)
  end
  
  # Check that DELETE to non-existent point gets 404
  test "destroy_404" do
    point = points(:wilf_dulce)
    http_auth_as(point.path.user.name)
    delete :destroy, :user_id => point.path.user.name, :path_id => point.path.alias, :id => point.path.next_position
    assert_response_code 404
  end
                        
end
