require 'test_helper'

# TODO: Test unexpected HTTP methods against RESTful interface, e.g. PUT /users
# TODO: Test various input formats for PUT/POST, e.g. XML -- in integration tests?
# TODO: Test direct GET requests to controller actions fail, e.g. /users/show, /users/delete
# TODO: Move test data out into config file or fixtures
# TODO: Test that sending XML with wrong root element fails
# TODO: Test that sending unknown format fails with 415
# TODO: Test admin user functionality
class UsersControllerTest < ActionController::TestCase
  # Getting user records returns 200 status code
  test "show_200" do
    bob = users(:bob)
    
    # Check that basic user record returned on unauthenticated request
    get :show, :id => bob.name, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'user', ['name'])
    
    # Check that full user record returned when HTTP Basic auth submitted
    http_auth_basic(bob.name, 'secret!')
    get :show, :id => bob.name, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'user', ['name', 'email', 'full-name', 'admin'])
  end
  
  # Request for a non-existent user returns 404
  test "show_404" do
    get :show, :id => 'nobody'
    assert_response_code 404
  end
  
  # Check that creating a new user gets 201
  test "create_201" do
    put :update, {:id => 'tester', :user => {:full_name => 'Tester', :email => 'tester@example.com', :password => 'secret!'}}
    assert_response_code 201
    assert_valid_xml_response(@response, 'user', ['name', 'email', 'full-name', 'admin'])
    assert_equal user_url('tester'), @response.headers['Location']
    assert User.find_by_name('tester')
  end
  
  # Check that POST to /users gets 201,
  # creating a new resource with name in request body
  test "post_create_201" do
    post :create, { :user => { :name => 'tester', :full_name => 'Tester', :email => 'tester@example.com', :password => 'secret!' } }
    assert_response_code 201
    assert_valid_xml_response(@response, 'user', ['name', 'email', 'full-name', 'admin'])
    assert_equal user_url('tester'), @response.headers['Location']
    assert User.find_by_name('tester')
  end
    
  # Creating a user with invalid data should return status 400
  test "create_400" do
    put :update, {:id => 'tester', :format => 'xml'} # i.e. incomplete user data
    assert_response_code 400
    assert_valid_xml(@response.body)
    assert !User.find_by_name('tester')
    # TODO: test that response body contains expected error messages 
  end
  
  # Trying to create a new account with an existing user name should return status 409.
  # Also triggered by attempting to update a user account without providing
  # authentication credentials.
  test "create_409" do
    put :update, {:id => users(:bob).name}, :format => 'xml'
    assert_response_code 409
    assert_valid_xml(@response.body)
    # TODO: test that response body contains expected error messages 
  end
  
  # Fail to update if authenticated as different user
  test "update_403" do
    admin = users(:admin)
    http_auth_basic(users(:bob).name, 'secret!')
    put :update, {:id => admin.name, :user => {:full_name => 'New name'}}
    assert_response_code 403
    assert_equal admin.full_name, User.find(admin.id).full_name, 'User update should have failed but succeeded'
  end

  # Updating account succeeds with appropriate authentication
  test "update_200" do
    bob = users(:bob)
    http_auth_basic(bob.name, 'secret!')
    put :update, {:id => bob.name, :user => {:full_name => 'New name'}}, :format => 'xml'
    assert_response_code 200
    assert_valid_xml_response(@response, 'user', ['name', 'email', 'full-name', 'admin'])
    assert_equal 'New name', User.find(bob.id).full_name, 'User update should have succeeded but failed'
  end
  
  # Succeed in update if authenticated as different admin user
  test "admin_update_200" do
    bob = users(:bob)
    http_auth_basic(users(:admin).name, 'admin!')
    put :update, {:id => bob.name, :user => {:full_name => 'New name', :admin => true}}
    assert_response_code 200
    assert_equal 'New name', User.find(bob.id).full_name, 'Other user update by admin should have succeeded but failed'
    assert_equal true, User.find(bob.id).admin, 'Admin should be able to make a non-admin user into an admin'
  end
  
  # Non-admin user should not be able to make themselves an admin
  test "admin_status_protected" do
    bob = users(:bob)
    http_auth_basic(bob.name, 'secret!')
    put :update, {:id => bob.name, :user => {:admin => true}} 
    assert_response_code 403 
    assert_equal false, User.find(bob.id).admin, 'Non-admin user should not be able to make themselves an admin'
  end
  
  # Last admin user should not make themselves a non-admin
  test "preserve_last_admin" do
    admin = users(:admin)
    http_auth_basic(admin.name, 'admin!')
    put :update, {:id => admin.name, :user => {:admin => false}} 
    assert_response_code 400
    assert_equal true, User.find(admin.id).admin, 'Last admin user should not be able to make themselves a non-admin'
  end
  
  # Attempt to change user name to a different, existing name triggers 409 status
  test "update_409" do
    bob = users(:bob)
    http_auth_basic(bob.name, 'secret!')
    put :update, {:id => bob.name, :user => {:name => users(:admin).name}}, :format => 'xml'
    assert_response_code 409
    assert_valid_xml(@response.body)
    # TODO: test that response body contains expected error messages 
  end
  
  # Check that DELETE authenticated & authorized succeeds with 200 status
  test "destroy_200" do
    bob = users(:bob)
    http_auth_basic(bob.name, 'secret!')
    delete :destroy, {:id => bob.name}
    assert_response_code 200
    assert !User.find_by_id(bob.id), 'User record should have been destroyed but still exists in db'
    delete :destroy, {:id => bob.name}
    assert_response_code 404
    # TODO: Enable XML validation check when controller updated to output description of destroyed object
    # assert_valid_xml(@response.body)
    # TODO: test that response body contains expected error messages 
  end
  
  # Check that DELETE unauthenticated fails with 401 status
  test "destroy_401" do
    bob = users(:bob)
    delete :destroy, {:id => bob.name}
    assert_response_code 401
    assert User.find_by_id(bob.id), 'User record should not been destroyed but is no longer in database'
  end
  
  # Check that DELETE authenticated but unauthorized fails with 403 status
  test "destroy_403" do
    admin = users(:admin)
    http_auth_basic(users(:bob).name, 'secret!')
    delete :destroy, {:id => admin.name}
    assert_response_code 403
    assert User.find_by_id(admin.id), 'User record should not been destroyed but is no longer in database'
  end
  
  # Check that DELETE other authenticated as admin succeeds 
  test "admin_destroy_200" do
    bob = users(:bob)
    http_auth_basic(users(:admin).name, 'admin!')
    delete :destroy, {:id => bob.name}
    assert_response_code 200
    assert !User.find_by_id(bob.id), 'User record should have been destroyed but is still in database'
  end
end
