require 'test_helper'

class PathTest < ActiveSupport::TestCase
  # Required attributes
  test "invalid_with_empty_attributes" do
    path = Path.new
    assert !path.valid?
    [:user_id, :name, :public].each do |column|
      assert path.errors.invalid?(column), "Expected empty #{column} to be invalid; #{column} = #{path.send(column).inspect}."
    end
  end
  
  # Certain names are reserved to prevent URI namespace clashes
  test "reserved_names" do
    Path::RESERVED_NAMES.each do |reserved|
      path = Path.new
      path.name = reserved
      assert !path.valid?
      assert_equal Path.default_error_messages[:reserved_name], path.errors.on(:name)
    end
  end
  
  # Path name should be unique for its parent user
  test "unique_name_for_path" do
    path = Path.new(:user_id => users(:bob).id, :name => users(:bob).paths.first.name, :public => true)
    assert !path.valid?
    assert_equal Path.default_error_messages[:unique_name], path.errors.on(:name)
  end
  
  # Destroying a path should also destroy its child points
  test "destroy_path_destroys_points" do
    path = paths(:wilf_poems)
    points = path.points
    path.destroy
    points.each do |point|
      assert_equal nil, Point.find_by_id(point.id), "Point (ID = #{point.id}) should have been destroyed when parent path (ID = #{path.id}) destroyed."
    end
  end
  
  # ASCII control characters not allowed in path name
  test "control_characters_in_name" do
    path = Path.new
    error_message = Path.default_error_messages[:control_characters]
    ctrl_chars = ("\x00".."\x1f").to_a + ("\x7f".."\x9f").to_a
    ctrl_chars.each do |char|
      path.name = char
      path.valid?
      assert path.errors.on(:name).include?(error_message), "Expected error message: <\"#{error_message}\">"
    end
  end
  
  # Path name is limited to 100 characters
  test "name_length" do
    path = Path.new
    path.name = '12345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901'
    path.valid?
    assert path.errors.on(:name).include?(I18n.translate('activerecord.errors.messages.too_long', :count => 100))
  end
  
  # When the last point in a path is destroyed, its position should be
  # the next available for a new point
  test "next_position_after_destroy_point" do
    path = paths(:wilf_poems)
    point = path.points.find(:all, :order => 'position').last
    point.destroy
    assert_equal point.position, path.next_position, "Next position in path should have decreased to destroyed point's position"
  end
  
  # User can not be changed once path saved
  test "constant_user_id" do
    path = users(:bob).paths.first
    path.user_id = users(:admin).id
    path.valid?
    assert_equal Path.default_error_messages[:constant_user_id], path.errors.on(:user_id), 'Parent user should not be changeable after path saved'
  end
  
  # Valid path should be saved
  test "valid" do
    path = Path.new(:name => 'New path', :user_id => users(:bob).id, :description => 'A brand new path', :public => true, :category_ids => [categories(:general).id] )
    assert path.save
    path = Path.find_by_id(path.id)
  end
end
