require 'test_helper'

class PointTest < ActiveSupport::TestCase
  def setup
    @point_template = Point.new(:uri => 'http://www.example.com/', :name => 'Point test')
  end
  
  # Required attributes
  test "invalid_with_empty_attributes" do
    point = Point.new
    assert !point.valid?
    [:path_id, :uri, :name].each do |column|
      assert point.errors.invalid?(column), "Column should not be valid: #{column} (value = #{point.read_attribute(column).inspect})"
    end
  end
  
  # URIs should start with http:// or https://
  test "http_uri" do
    valid_uris = ['http://www.example.com/', 'http://', 'https://sub.example.com/path/to/resource.txt']
    invalid_uris = ['www.example.com', 'ftp://ftp.example.com']
    
    point = Point.new
    
    valid_uris.each do |valid_uri|
      point.uri = valid_uri
      point.valid?
      assert !point.errors.invalid?(:uri), "URI should be valid: #{valid_uri}"
    end
    
    invalid_uris.each do |invalid_uri|
      point.uri = invalid_uri
      point.valid?
      assert_equal Point.default_error_messages[:uri_format], point.errors.on(:uri), "URI should be invalid: #{invalid_uri}"
    end
  end
  
  # Invalid values for position should not be permitted
  test "invalid_positions" do
    path = paths(:wilf_poems)
    bad = [-1, 0, 'one']
    bad.each do |position|
      point = @point_template.clone
      point.path_id = path.id
      point.position = position
      point.valid?
      assert point.errors.invalid?(:position), "Position #{position.inspect} should not be valid"
    end
  end
  
  # Position should be auto-assigned to next available in path if not specified
  test "auto_position" do
    # Test that next available position is used
    path = paths(:wilf_poems)
    next_position = path.next_position
    point = @point_template.clone
    point.path_id = path.id
    assert point.save, point.errors.full_messages
    assert_equal next_position, point.position
    
    # Test that position auto-assigned is 1 if no existing points in path
    path = Path.create(:user_id => users(:bob).id, :name => 'Parent path for first auto-assigned point', :public => true)
    point = @point_template.clone
    point.path_id = path.id
    assert point.save
    assert_equal 1, point.position
    
    # Test that the next has position 2
    point = @point_template.clone
    point.path_id = path.id
    assert point.save
    assert_equal 2, point.position
  end
    
  # Path can not be changed once point saved
  test "constant_path_id" do
    point = paths(:wilf_poems).points.first
    point.path_id = paths(:oucs_services).id
    point.valid?
    assert_equal Point.default_error_messages[:constant_path_id], point.errors.on(:path_id), 'Parent path should not be changeable after point saved'
  end
end
