require 'test_helper'

class UserTest < ActiveSupport::TestCase
  # Required attributes
  test "invalid_with_empty_attributes" do
    ['', nil].each do |value|
      [:name, :full_name, :email, :password].each do |column|
        user = User.new
        user[column] = value
        assert !user.valid?
        assert [user.errors.on(column)].flatten.include?(I18n.translate('activerecord.errors.messages.blank'))
      end
    end
  end
  
  # User name needs to be unique
  test "unique_name" do
    user = User.new(:name => users(:bob).name)
    user.valid?
    assert_equal User.default_error_messages[:unique_name], user.errors.on(:name)
  end

  # Format of user name
  test "name_format" do
    user = User.new
    
    valid_names = [ 'adam.smith', 'adamsmith', 'adam1', 'adamsmith1969' ]
    valid_names.each do |name|
      user.name = name
      user.valid?
      assert !user.errors.invalid?(:name)
    end
    
    invalid_names = { 
      'ab' => I18n.translate('activerecord.errors.messages.too_short', :count => 3),
      'abcdefghijabcdefghijabcdefghijabc' => I18n.translate('activerecord.errors.messages.too_long', :count => 32),
      'ab-cd' => User.default_error_messages[:name_valid_characters],
      'abcdef!gh' => User.default_error_messages[:name_valid_characters],
      'adam "smithy" smith' => User.default_error_messages[:name_valid_characters],
      'abCd' => User.default_error_messages[:name_valid_characters],
      '123456' => User.default_error_messages[:name_first_character],
      'abcd.' => User.default_error_messages[:name_last_character],
      'ab..cd' => User.default_error_messages[:no_consecutive_periods]
    }
    invalid_names.each_pair do |name, error_message|
      user.name = name
      user.valid?
      assert user.errors.invalid?(:name)
      assert [user.errors.on(:name)].flatten.include?(error_message), "Username: #{name}\nExpected error message: <\"#{error_message}\">"
    end
    
  end
  
  # Certain names are reserved to prevent URI namespace clashes
  test "reserved_names" do
    User::RESERVED_NAMES.each do |reserved|
      user = User.new
      user.name = reserved
      assert !user.valid?
      assert_equal User.default_error_messages[:reserved_name], user.errors.on(:name)
    end
  end
  
  # Destroying a user should destroy its child paths
  test "destroy_user_destroys_paths" do
    user = users(:bob)
    paths = user.paths
    user.destroy
    paths.each do |path|
      assert_equal nil, Path.find_by_id(path.id), "Path (ID = #{path.id}) should have been destroyed when parent user (ID = #{user.id}) destroyed."
    end
  end
  
  # Password should be hashed
  test "password_hashed" do
    plain_password = '123456'
    user = User.new
    user.password = plain_password
    user.valid?
    assert_equal user.password, plain_password, 'Password comparison should succeed'
    assert_not_equal plain_password, user.password, 'Password should not be set in its unencrypted state'
  end
    
  # Common password is disallowed
  test "common_password" do
    user = User.new
    user.password = '123456'
    user.valid?
    assert_equal User.default_error_messages[:common_password], user.errors.on(:password)
  end
  
  # Random, secure password is OK
  test "secure_password" do
    user = User.new
    user.password = 'secret!'
    user.valid?
    assert !user.errors.invalid?(:password), 'Secure password should be OK'
  end
    
  # Password must be at least 6 characters
  test "password_length  " do
    user = User.new
    user.password = '!'
    user.valid?
    assert_equal I18n.translate('activerecord.errors.messages.too_short', :count => 6), user.errors.on(:password)
  end
  
  # Password should not be the same as username
  test "password_matches_username" do
    user = User.new
    user.name = 'tester'
    user.password = 'tester'
    user.valid?
    assert [user.errors.on(:password)].flatten.include?(User.default_error_messages[:password_equals_name])
  end
end
